//
// Scheduler.h
//
// Copyright (c) Shareaza Development Team, 2002-2007.
// This file is part of SHAREAZA (shareaza.sourceforge.net)
//
// Shareaza is free software; you can redistribute it
// and/or modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2 of
// the License, or (at your option) any later version.
//
// Shareaza is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with Shareaza; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//
#if !defined(AFX_SCHEDULER_H__INCLUDED_)
#define AFX_SCHEDULER_H__INCLUDED_

#pragma once

//TODO: Add new tasks here
enum ScheduleTask
{
	SCHEDULE_FULL_SPEED, SCHEDULE_LIMITED_SPEED, SCHEDULE_OFF, SCHEDULE_DISCONNECT, SCHEDULE_EXIT, SCHEDULE_SHUTDOWN  
};

//////////////////////////////////////////////////////////////////////
// CScheduleItem class: Represents a scheduled task
//////////////////////////////////////////////////////////////////////

class CScheduleItem
{
// Construction
public:
	CScheduleItem(BOOL bCreate = TRUE);
	CScheduleItem(const CScheduleItem& pItem);
	virtual ~CScheduleItem();

// Attributes
public:
	int			m_nAction;			//Will have one of ScheduleTask values plus -1 as invalid state indicator
	BOOL		m_bEveryDay;		//Task is scheduled for everyday or just today
	CString		m_sDescription;		//Optional task description
	CTime		m_tScheduleDateTime;//Time the task is scheduled for
	CString		m_sDate;			//String representaion of date in m_tScheduleDateTime
	CString		m_sTime;			//String representaion of clock in m_tScheduleDateTime
	BOOL		m_bActive;			//Task should be executed or not
	BOOL		m_bExecuted;		//Task is executed or not
	BOOL		m_bToggleBandwidth;	//Up/Down bandwidth are limited seperately or not
	BOOL		m_bLimitedNetworks;	//Network is limited to G2 or not (in SCHEDULE_LIMITED_SPEED)
	int			m_nLimited;			//Bandwidth limit when m_bToggleBandwidth is FALSE
	int			m_nLimitedDown;		//Down stream bandwidth limit when m_bToggleBandwidth is TRUE
	int			m_nLimitedUp;		//Up stream bandwidth limit when m_bToggleBandwidth is TRUE
	GUID		m_pGUID;			//GUID for each scheduled item

// Operations
public:
	void		Serialize(CArchive& ar, int nVersion);
};

//////////////////////////////////////////////////////////////////////
// CScheduler class: Controls scheduler operations
//////////////////////////////////////////////////////////////////////

class CScheduler  
{
// Construction
public:
	CScheduler();
	virtual ~CScheduler();
	
// Attributes
protected:
	CList< CScheduleItem* >	m_pScheduleItems;

// Operations
public:
	//Lock is used when objects reads/wirtes from/to m_pScheduleItems 
	mutable CCriticalSection		m_pSection;
	
	//To iterate through m_pScheduleItems
	POSITION		GetIterator() const;
	CScheduleItem*	GetNext(POSITION& pos) const;
	int				GetCount() const;
	//Checks to see pItem exists in m_pScheduleItems or not, by comparing GUID values
	BOOL			Check(CScheduleItem* pItem) const;
	
	//It is called regularly by timers to see if any scheduled item should be executed
	//This method also sets Settings.Scheduler.Enable to indiate globally if any item 
	//is going to be executed
	void			CheckSchedule();

	//Is used to disconnect dial up connection
	void			HangUpConnection();

	//Is used to shut down computer
	BOOL			ShutDownComputer();

	//Is called by Load(). Tries to get shutdown privilege for the process
	BOOL			SetShutdownRights();
	
	//Is called by CheckSchedule(). Checks to see if Now is grater than Then or not.
	BOOL			ScheduledTimePassed(CTime *pNow,CTime *pThen);
	
	//Adds a new task to m_pScheduleItems after giving it a GUID
	void			Add(CScheduleItem* pItem);
	
	//Removes a task from m_pScheduleItems if it exists in the m_pScheduleItems
	void			Remove(CScheduleItem* pItem);
	
	//Clears all m_pScheduleItems items
	void			Clear();

	BOOL			Load();
	BOOL			Save();

	//Calculates hours remaining to system shutdown or disconnect based on scheduled tasks
	int				GetHoursToShutdown();

protected:
	void			Serialize(CArchive& ar);

	//Called by CheckSchedule() to execute a task
	void			DoScheduledItem(CScheduleItem* pItem);

	CScheduleItem*	GetGUID(const GUID& pGUID) const;
};

extern CScheduler Scheduler;

#endif // !defined(AFX_SCHEDULER_H__INCLUDED_)
